<?php

/* ---- Tags Support ---- */
// Generate a Tag cloud
function dw_tagged_cloud() {
	$font_size_min = 16; $font_size_max = 48;  // Smallest Font Size in px -> Largest Font Size in px

    $tag_count = array();  // Array to keep track of Count of Tags
    $tag_arr = array();  // Array to keep track of Tags and Font Sizes

	// Get List of All Tags - Pages
    $query_pages = Query::table(Base::table('page_meta'))
        ->join(Base::table('extend'), Base::table('extend.id'), '=', Base::table('page_meta.extend'))
        ->where(Base::table('extend.key'), '=', 'tags');
    $tags_pages = $query_pages->get(array(Base::table('page_meta.data')));
	// Get List of All Tags - Posts
    $query_posts = Query::table(Base::table('post_meta'))
        ->join(Base::table('extend'), Base::table('extend.id'), '=', Base::table('post_meta.extend'))
        ->where(Base::table('extend.key'), '=', 'tags');
    $tags_posts = $query_posts->get(array(Base::table('post_meta.data')));

	// Count The number of Articles using Each Tag - Pages
    foreach(array_keys($tags_pages) as $tags_index) {
        $tags_this = '';
        $tags_this = Json::decode($tags_pages[$tags_index] ? $tags_pages[$tags_index]->data : '{}');
        if( ! empty($tags_this->text)) {
            $tag_current = $tags_this->text;
            $tag_current_arr = explode(",", $tag_current);
            foreach ($tag_current_arr as $taggy) {
            	$taggy = trim($taggy);
            	if (!isset($tag_count[$taggy])) {
            		$tag_count[$taggy] = 0;
            	}
            	$tag_count[$taggy] = $tag_count[$taggy] + 1;
            }
        }
    }
	// Count The number of Articles using Each Tag - Posts
    foreach(array_keys($tags_posts) as $tags_index) {
        $tags_this = '';
        $tags_this = Json::decode($tags_posts[$tags_index] ? $tags_posts[$tags_index]->data : '{}');
        if( ! empty($tags_this->text)) {
            $tag_current = $tags_this->text;
            $tag_current_arr = explode(",", $tag_current);
            foreach ($tag_current_arr as $taggy) {
            	$taggy = trim($taggy);
            	if (!isset($tag_count[$taggy])) {
            		$tag_count[$taggy] = 0;
            	}
            	$tag_count[$taggy] = $tag_count[$taggy] + 1;
            }
        }
    }

	// Calculate Font Sizes
	if (! empty($tag_count)) {
		$min_count = min($tag_count);
		$spread = max($tag_count) - $min_count;
		if ($spread <= 0) { $spread = 1; }
		$font_spread = $font_size_max - $font_size_min;
		if ($font_spread <= 0) { $font_spread = 1; }
		$font_step = $font_spread / $spread;

		// Assign Font Size to Each Tag
		foreach($tag_count as $taggy => $tagcount) {
			if ($tagcount > 0) {
				$tag_arr[$taggy] = $font_size_min + (($tagcount - $min_count) * $font_step);
			}
		}
	}
	ksort($tag_arr);  // Sort by Tag Name
    // arsort($tag_arr); // Sort by Tag Count
    return $tag_arr;
}

// Checks the number of Results in a Tag Search, after processing by <anchor>/models/post.php & page.php
function dw_has_tagged_results_all() {
    return Registry::get('total_all', 0) > 0;
}
function dw_has_tagged_results_pages() {
    return Registry::get('total_pages', 0) > 0;
}
function dw_has_tagged_results_posts() {
    return Registry::get('total_posts', 0) > 0;
}

// Gets the number of Results in a Tag Search, after processing by <anchor>/models/post.php & page.php
function dw_total_tagged_results_all() {
    return Registry::get('total_all', 0);
}
function dw_total_tagged_results_pages() {
    return Registry::get('total_pages', 0);
}
function dw_total_tagged_results_posts() {
    return Registry::get('total_posts', 0);
}

// Returns the List of Articles and Pages with the Requested Tag, after processing by <anchor>/models/post.php & page.php
function dw_tagged_results_pages() {
    $pages = Registry::get('tagged_results_pages');
	if($result = $pages->valid()) {
		Registry::set('page', $pages->current());  // register single page
		$pages->next();  // move to next
	}
    return $result;
}
function dw_tagged_results_posts() {
    $posts = Registry::get('tagged_results_posts');
    if($result = $posts->valid()) {
        Registry::set('article', $posts->current());  // register single post
        $posts->next();  // move to next
    }
    return $result;
}

// Returns the Tag Term Requested, after processing by <anchor>/models/post.php & page.php
function dw_tagged_term() {
    return Registry::get('tagged_term');
}

// Checks if this Tag Listing Needs Pagination, after processing by <anchor>/models/post.php & page.php
function dw_has_tagged_pagination() {
    return Registry::get('total_max') > Config::meta('posts_per_page');
}

function dw_tagged_next($text = 'Next', $default = '') {
    $per_page = Config::meta('posts_per_page');
    $page = Registry::get('page_offset');

    $offset = ($page - 1) * $per_page;
    $total = Registry::get('total_max');

    $pages = floor($total / $per_page);

    $tagged_page = Registry::get('page');
    $next = $page + 1;
    $term = Registry::get('tagged_term');
    Session::put(slug($term), $term);

    $url = base_url('tags/' . $term . '/' . $next);

    if(($page - 1) < $pages) {
        return '<a href="' . $url . '">' . $text . '</a>';
    }

    return $default;
}

function dw_tagged_prev($text = 'Previous', $default = '') {
    $per_page = Config::get('meta.posts_per_page');
    $page = Registry::get('page_offset');

    $offset = ($page - 1) * $per_page;
    $total = Registry::get('total_max');

    $pages = ceil($total / $per_page);

    $tagged_page = Registry::get('page');
    $prev = $page - 1;
    $term = Registry::get('tagged_term');
    Session::put(slug($term), $term);

    $url = base_url('tags/' . $term . '/' . $prev);

    if($offset > 0) {
        return '<a href="' . $url . '">' . $text . '</a>';
    }

    return $default;
}

// Returns a URL for a Tag Link
function dw_tagged_url() {
    return base_url('tags');
}

function dw_tagged_form_input($extra = '') {
    return '<input name="term" type="text" ' . $extra . ' value="' . dw_tagged_term() . '">';
}

/* ---- Other Functions ---- */
function dw_pluralise($amount, $str, $alt = '') {
	return intval($amount) === 1 ? $str : $str . ($alt !== '' ? $alt : 's');
}

// Estimated Reading Time
function dw_estimated_reading_time($wordcount) {
	$m = floor($wordcount / 175);
	$s = floor($wordcount % 175 / (175 / 60));
	if($m >= 1) {
		return $est = $m . ' min' . ($m == 1 ? '' : 's');
	} elseif($s <= 59) {
		return $est = $s . ' second' . ($s == 1 ? '' : 's');
	}
}

// Relative Time
function dw_relative_time($date) {
	if(is_numeric($date)) $date = '@' . $date;

	$user_timezone = new DateTimeZone(Config::app('timezone'));
	$date = new DateTime($date, $user_timezone);

	// get current date in user timezone
	$now = new DateTime('now', $user_timezone);

	$elapsed = $now->format('U') - $date->format('U');

	if($elapsed <= 1) {
		return 'Just now';
	}

	$times = array(
		31104000 => 'year',
		2592000 => 'month',
		604800 => 'week',
		86400 => 'day',
		3600 => 'hour',
		60 => 'minute',
		1 => 'second'
	);

	foreach($times as $seconds => $title) {
		$rounded = $elapsed / $seconds;

		if($rounded > 1) {
			$rounded = round($rounded);
			return $rounded . ' ' . dw_pluralise($rounded, $title) . ' ago';
		}
	}
}

// Generate Social Media Sharing Buttons
function dw_gensharelink() {
	// This is our current URL
	$sURL = urlencode(
            'http' .
            ($_SERVER['SERVER_PORT'] == 443 ? 's' : '') . '://' .
            $_SERVER['HTTP_HOST'] .
            $_SERVER['PHP_SELF']
        );
    if (strlen($sURL) < 1) { return ""; }
    // Get other document properties
    $doc_title = article_title();
    if (strlen($doc_title) == 0) {
	    $doc_title = page_title();
	}
	$doc_desc = article_description();
	$doc_tags = "";
	$tagcount = 0;
	if (article_custom_field('tags')) {
		$tagarray = array();
		$tagexplode = explode(",", article_custom_field('tags'));
		foreach ($tagexplode as $taggy) {
			$taggy = trim($taggy);
			if (0 < $tagcount) { $doc_tags = $doc_tags . ","; }
			$doc_tags = $doc_tags . $taggy;
			$tagcount = $tagcount + 1;
		}
	}
	if (page_custom_field('tags')) {
		$tagarray = array();
		$tagexplode = explode(",", page_custom_field('tags'));
		foreach ($tagexplode as $taggy) {
			$taggy = trim($taggy);
			if (0 < $tagcount) { $doc_tags = $doc_tags . ","; }
			$doc_tags = $doc_tags . $taggy;
			$tagcount = $tagcount + 1;
		}
	}
	$sdoc_tags = urlencode($doc_tags);
	$twittags = "";
	$linkedtitle = "";
	$linkedsource = "";
	$linkeddesc = "";
	if (strlen($sdoc_tags) > 0) { $twittags = "&hashtags=" . $sdoc_tags; }
	if (strlen($doc_title) > 0) { $linkedtitle = "&title=" . urlencode($doc_title); }
	if (strlen(site_name()) > 0) { $linkedsource = "&source=" . urlencode(site_name()); }
	if (strlen($doc_desc) > 0) { $linkeddesc = "&summary=" . urlencode($doc_desc); }
	// Generate the Links
    $fblink = "<a href=\"https://www.facebook.com/sharer/sharer.php?u=".$sURL."\" class=\"socialmedia\">f</a>\n";
    $pluslink = "<a href=\"https://plus.google.com/share?url=".$sURL."\" class=\"socialmedia\">g</a>\n";
    $twitlink = "<a href=\"https://twitter.com/intent/tweet?url=".$sURL.$twittags."\" class=\"socialmedia\">t</a>\n";
    $linkedlink = "<a href=\"http://www.linkedin.com/shareArticle?mini=true&url=".$sURL.$linkedtitle.$linkedsource.$linkeddesc."\" class=\"socialmedia\">l</a>\n";
    return "Share this article: &nbsp; " . $fblink . $pluslink . $twitlink . $linkedlink;
}
